#include "3DTestScene.h"
#include "DirectX.h"
#include "DrawBuffer.h"
#include "Texture.h"

#define FVF_CUSTOM3D    (D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1)

struct SimpleVertex
{
	float x, y, z;
	DWORD color;
	float u, v;
};

static TextureData g_texData;
static IDirect3DVertexBuffer9* g_pVertexBuffer;

static D3DXMATRIX g_ViewMatrix;
static D3DXMATRIX g_ProjectionMatrix;
static D3DXMATRIX g_WorldMatrix;

// 
static bool SceneInit(void)
{
	// assetstH_bridge.pngeNX`[Ƃēǂݍ
	if( !LoadTexture(TEXT("assets/bridge.png"), &g_texData) )
	{
		return false;
	}
	float aspect = (float)g_texData.srcWidth / (float)g_texData.srcHeight;
	// 摜TCY畝ƍݒ
	float rect[] =
	{
		-0.5f * aspect,  0.5f,
		 0.5f * aspect, -0.5f,
	};
	// eNX`[TCY摜TCYUV擾(摜2̗ݏł1.0fɂȂ)
	float u = (float)g_texData.srcWidth / (float)g_texData.width;
	float v = (float)g_texData.srcHeight / (float)g_texData.height;
	// 摜TCYɍ킹UVǉ_f[^
	const SimpleVertex vertices[4] =
	{
		// 1ڂ̋`
		{ rect[0], rect[1], 0.0f, 0xffffffff, 0.0f, 0.0f },
		{ rect[2], rect[1], 0.0f, 0xffffffff,    u, 0.0f },
		{ rect[0], rect[3], 0.0f, 0xffffffff, 0.0f,    v },
		{ rect[2], rect[3], 0.0f, 0xffffffff,    u,    v },
	};
	// _obt@̐
	g_pVertexBuffer = CreateVertexBuffer(vertices, sizeof(vertices));
	if( g_pVertexBuffer == NULL )
	{
		return false;
	}
	return true;
}

// 
static void SceneFinal(void)
{
	ReleaseTexture(&g_texData);
	SAFE_RELEASE(g_pVertexBuffer);
}

// XV
static void SceneUpdate(void)
{
	D3DXMatrixPerspectiveFovLH(
		&g_ProjectionMatrix,
		D3DXToRadian(45.0f),
		640.0f / 480.0f,
		0.01f,
		1000.0f
	);

	D3DXVECTOR3 cameraPos(0.0f, 0.5f, -3.0f);
	D3DXVECTOR3 targetPos(0.0f, 0.0f, 0.0f);
	D3DXVECTOR3 upVec(0.0f, 1.0f, 0.0f);
	D3DXMatrixLookAtLH(&g_ViewMatrix, &cameraPos, &targetPos, &upVec);

	static float angle = 0.0f;
	angle += D3DXToRadian(1.0f);
	D3DXMatrixRotationY(&g_WorldMatrix, angle);
}

// `
static void SceneRender(void)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();

	// View, Projections̐ݒ
	pDevice->SetTransform(D3DTS_VIEW, &g_ViewMatrix);
	pDevice->SetTransform(D3DTS_PROJECTION, &g_ProjectionMatrix);

	// ʏ폑݃[hɐݒ
	SetRenderMode(ERenderMode::Normal, false);

	pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	// Worlds̐ݒ
	pDevice->SetTransform(D3DTS_WORLD, &g_WorldMatrix);
	// eNX`[̐ݒ
	pDevice->SetTexture(0, g_texData.pTexture);
	// _obt@̐ݒ
	pDevice->SetStreamSource(0, g_pVertexBuffer, 0, sizeof(SimpleVertex));

	// _tH[}bg̎w
	pDevice->SetFVF(FVF_CUSTOM3D);
	// `
	pDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
}

// V[̐ݒ
SceneProc Setup3DTestScene(void)
{
	SceneProc proc =
	{
		"3DTest",
		SceneInit,
		SceneFinal,
		SceneUpdate,
		SceneRender,
	};
	return proc;
}