#include "DirectX.h"
#include "Window.h"

#pragma comment(lib, "d3d9.lib")
#pragma comment(lib, "d3dx9.lib")

static IDirect3D9*			g_pD3D = NULL;
static IDirect3DDevice9*	g_pDevice = NULL;

// DirectX̏
bool InitializeDirectX(HWND hWnd, int width, int height, bool isFullscreen)
{
	if( g_pD3D != NULL || g_pDevice != NULL ) return false;

	// IDirect3D9IuWFNg̍쐬
	if( (g_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL )
	{
		return false;
	}

	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(D3DPRESENT_PARAMETERS));
	// Device̐ݒ
	d3dpp.Windowed						= !isFullscreen;
	d3dpp.hDeviceWindow					= hWnd;
	d3dpp.BackBufferWidth				= width;
	d3dpp.BackBufferHeight				= height;
	d3dpp.BackBufferFormat				= D3DFMT_UNKNOWN;
	d3dpp.SwapEffect					= D3DSWAPEFFECT_DISCARD;
	d3dpp.MultiSampleType				= D3DMULTISAMPLE_8_SAMPLES;
	d3dpp.MultiSampleQuality			= 0;
	d3dpp.Flags							= 0;
	d3dpp.BackBufferCount				= 1;
	d3dpp.EnableAutoDepthStencil		= true;
	d3dpp.AutoDepthStencilFormat		= D3DFMT_D24S8;
	// VSYNC҂L)
	d3dpp.PresentationInterval			= D3DPRESENT_INTERVAL_ONE;
	d3dpp.FullScreen_RefreshRateInHz	= D3DPRESENT_RATE_DEFAULT;

	// Direct3DfoCX̐
	if( FAILED(g_pD3D->CreateDevice(D3DADAPTER_DEFAULT,
		D3DDEVTYPE_HAL,
		d3dpp.hDeviceWindow,
		D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED,
		&d3dpp,
		&g_pDevice)) )
	{
		SAFE_RELEASE(g_pD3D);
		return false;
	}
	return true;
}

// DirectX̏I
void FinalizeDirectX(void)
{
	SAFE_RELEASE(g_pDevice);
	SAFE_RELEASE(g_pD3D);
}

// Direct3DIuWFNg̎擾
IDirect3D9* GetDirect3D(void) { return g_pD3D; }
// Direct3DDeviceIuWFNg̎擾
IDirect3DDevice9* GetDirect3DDevice(void) { return g_pDevice; }


// `V[̊Jn
void BeginRenderScene(void)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();
	if( pDevice ) pDevice->BeginScene();
}

// `V[̏I
void EndRenderScene(void)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();
	if( pDevice ) pDevice->EndScene();
}

// obNobt@̃NA
void ClearBackBuffer(DWORD target, D3DCOLOR color, float depth, DWORD stencil)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();
	if( pDevice ) pDevice->Clear(0, NULL, target, color, depth, stencil);
}

// ʂ̃tbv
void FlipDisp(void)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();
	if( pDevice ) pDevice->Present(NULL, NULL, NULL, NULL);
}

// ̕`Xe[gݒ
void SetDefaultRenderState(void)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();
	// JO[h
	pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	// tB[h
	pDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	// Zobt@Lɂ邩H
	pDevice->SetRenderState(D3DRS_ZENABLE, true);
	// Zobt@݂̏Lɂ邩H
	pDevice->SetRenderState(D3DRS_ZWRITEENABLE, true);
	// ZeXg̔r@
	pDevice->SetRenderState(D3DRS_ZFUNC, D3DCMP_ALWAYS);
	// uhLɂ邩H
	pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	// eXgLɂ邩H
	pDevice->SetRenderState(D3DRS_ALPHATESTENABLE, true);
	// eXg̔@
	pDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_NOTEQUAL);
	// eXg̎Qƒl
	pDevice->SetRenderState(D3DRS_ALPHAREF, 0);

	//	`ݒ
	pDevice->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
	pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	//pDevice->SetRenderState(D3DRS_SHADEMODE, D3DSHADE_GOURAUD);
	//pDevice->SetRenderState(D3DRS_NORMALIZENORMALS, true);

	// J[Xe[Wݒ
	pDevice->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
	pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
	// Xe[Wݒ
	pDevice->SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
	pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	pDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_CURRENT);

	// eNX`[tB^[̐ݒ
	pDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
	pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	// eNX`[̃AhbVO[h̐ݒ
	pDevice->SetSamplerState(0, D3DSAMP_ADDRESSU,  D3DTADDRESS_WRAP);
	pDevice->SetSamplerState(0, D3DSAMP_ADDRESSV,  D3DTADDRESS_WRAP);
}

// `Xe[g̐ݒ
void SetRenderMode(ERenderMode mode, bool enableAlpha)
{
	IDirect3DDevice9* pDevice = GetDirect3DDevice();
	// ȏꍇ͓͂ꂽJ[̂܂܎g
	DWORD srcColorblend = D3DBLEND_ONE;
	DWORD dstColorblend = D3DBLEND_ZERO;
	if( enableAlpha )
	{
		// Lȏꍇ̓lJ[ɉe
		srcColorblend = D3DBLEND_SRCALPHA;
		dstColorblend = D3DBLEND_INVSRCALPHA;
		// eXgŃl0̏ꍇ͕`Ȃ悤ɐݒ
		pDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_NOTEQUAL);
		pDevice->SetRenderState(D3DRS_ALPHAREF, 0);
	}
	// uhƃeXg̗Lݒ
	pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, enableAlpha);
	pDevice->SetRenderState(D3DRS_ALPHATESTENABLE, enableAlpha);

	switch (mode)
	{
	case Normal:
		// LF(SrcColor * 1) + (DestColor * 0)
		// F(SrcColor * ) + (DestColor * (1 - ))
		pDevice->SetRenderState(D3DRS_BLENDOP,   D3DBLENDOP_ADD);
		pDevice->SetRenderState(D3DRS_SRCBLEND,  srcColorblend);
		pDevice->SetRenderState(D3DRS_DESTBLEND, dstColorblend);
		break;
	case Add:
		// LF(SrcColor * 1) + (DestColor * 1)
		// F(SrcColor * ) + (DestColor * 1)
		pDevice->SetRenderState(D3DRS_BLENDOP,   D3DBLENDOP_ADD);
		pDevice->SetRenderState(D3DRS_SRCBLEND,  srcColorblend);
		pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		break;
	case Subtract:
		// LF(DestColor * 1) - (SrcColor * 1)
		// F(DestColor * 1) - (SrcColor * )
		pDevice->SetRenderState(D3DRS_BLENDOP,   D3DBLENDOP_REVSUBTRACT);
		pDevice->SetRenderState(D3DRS_SRCBLEND,  srcColorblend);
		pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		break;
	case Multiple:
		// ʁF(SrcColor * 0) + (DestColor * SrcColor)
		pDevice->SetRenderState(D3DRS_BLENDOP,   D3DBLENDOP_ADD);
		pDevice->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ZERO);
		pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR);
		break;
	}
}